#define _USE_MATH_DEFINES

#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>
#include "..\buzz\MachineInterface.h"
#include "IMaitoPisaraMach.h"

#include "MaitoPisaraWnd.h"



CMachineParameter const paraPreset = { pt_word, "Preset", "Preset", 0,	65535, 255, MPF_STATE, 0 };
CMachineParameter const paraCommand = { pt_byte, "Command", "Command", 0, 255, 255, MPF_STATE, 0 };
CMachineParameter const paraValue = { pt_byte, "Value", "Value", 0, 255, 255, MPF_STATE, 0 };

CMachineParameter const *pParameters[] = { &paraCommand, &paraValue, &paraPreset };

#define MAITO_BUFFER_SIZE 576*2

#pragma pack(1)		
class gvals
{
public:
	byte threshold;
};
#pragma pack()

CMachineInfo const MacInfo = 
{
	MT_EFFECT,								// type
	MI_VERSION,	
	MIF_STEREO_EFFECT,
	0,										// min tracks
	0,										// max tracks
	3,										// numGlobalParameters
	0,										// numTrackParameters
	pParameters,
	0,
	NULL,
#ifdef _DEBUG
	"WDE MaitoPisara (Debug)",		// name
#else
	"WDE MaitoPisara",					// name
#endif
	"MaitoPisara",									// short name
	"WDE",						// author
	NULL
};

class mi;

class miex : public CMachineInterfaceEx
{
public:
	mi *pmi;
};

class mi : public CMachineInterface, IMaitoPisaraMach
{
public:
	mi();
	virtual ~mi();
	virtual void Init(CMachineDataInput * const pi);
	virtual void Tick();
	virtual bool Work(float *psamples, int numsamples, int const mode);

	bool silentinput;
	CMachine* thisMachine;

	float maitoBuffer[MAITO_BUFFER_SIZE];
	int maitoBufferIndex;
	int maitoBufferReadIndex;
	bool milkDropCreated;

	miex ex;	
	gvals gval;

	void getAudioBufferData(float *waves, int samplesPerChannel);
	void milkdropDeleted();
	
	bool DClickMachine(void *)
	{
		if (!milkDropCreated)
		{
			milkDropCreated = true;

			TCHAR NPath[MAX_PATH];
			GetCurrentDirectory(MAX_PATH, NPath);
			pCB->WriteLine(NPath);

			TCHAR fullPath[MAX_PATH];
			TCHAR driveLetter[3];
			TCHAR directory[MAX_PATH];
			TCHAR FinalPath[MAX_PATH];
			GetModuleFileName(NULL, fullPath, MAX_PATH);
			_splitpath(fullPath, driveLetter, directory, NULL, NULL);
			sprintf(FinalPath, "%s%s", driveLetter, directory);

			pCB->WriteLine(FinalPath);

			SetCurrentDirectory(FinalPath);
			MaitoPisaraInitWindow(pCB->GetMachineName(thisMachine), pCB->GetMachineName(thisMachine));
			MaitoPisaraWinMain(NULL, NULL, NULL, 0, this);
		}
		else
		{
			BringMaitoPisaraWndOnTop();
		}
		return true;
	}
};

DLL_EXPORTS

mi::mi()
{
	ex.pmi = this;
	GlobalVals = &gval;
	AttrVals = NULL;
	maitoBufferIndex = 0;
	maitoBufferReadIndex = 0;
	milkDropCreated = false;		
}

mi::~mi()
{
	BuzzClosed(true);
	SendMessage(gHWND, WM_QUIT, 0, 0);
}

void mi::Init(CMachineDataInput * const pi)
{
	pCB->SetMachineInterfaceEx(&ex);
	pCB->WriteLine("MaitoPisara");	
	pCB->SetEventHandler(pCB->GetThisMachine(), DoubleClickMachine, (EVENT_HANDLER_PTR)&mi::DClickMachine, NULL);
	for (int i = 0; i < MAITO_BUFFER_SIZE; i++)
	{
		maitoBuffer[i] = 0.0f;
	}
	thisMachine = pCB->GetThisMachine();
}

void mi::Tick()
{
}

bool mi::Work(float *psamples, int numsamples, int const mode)
{
	bool ret = true;

	silentinput = true; 
	float epsilon = 0.0000001f;

	for (int i = 0; i < numsamples; i++)
	{
		if (maitoBufferIndex >= MAITO_BUFFER_SIZE)
		{
			maitoBufferIndex = 0;
		}

		maitoBuffer[maitoBufferIndex] = psamples[i];
		maitoBufferIndex++;

	}

	for (int i=0; i<numsamples; i++) 
	{ 
	   if (psamples[i] > epsilon || psamples[i] < -epsilon) 
	   { 
		  silentinput = false; 
		  break; 
	   } 
	}

	if (mode & WM_READ)
	{

	}
	else
	{
		if (silentinput)
		{
			ret = false;
		}
		
		memset(psamples, 0, 2 * sizeof(float) * numsamples);
	}

	return ret;
}

void mi::getAudioBufferData(float *waves, int samplesPerChannel)
{
	for (int i = 0; i < samplesPerChannel*2; i++)
	{
		if (maitoBufferReadIndex >= MAITO_BUFFER_SIZE)
		{
			maitoBufferReadIndex = 0;
		}

		waves[i] = maitoBuffer[maitoBufferReadIndex];
		maitoBufferReadIndex++;
	}
}

void mi::milkdropDeleted()
{
	this->milkDropCreated = false;
}
